# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
import logging
from collections import OrderedDict

from flask import Blueprint, jsonify, render_template
from flask_restplus import fields
from flask.ext.restplus import Resource, Api, apidoc

from pyris.api import extract


Logger = logging.getLogger(__name__)

service = Blueprint(
    'api',
    __name__,
    url_prefix='/pyris'
)

@service.route('/')
def index():
    return render_template("index.html")


api = Api(service,
          title='IRIS Geolocalization',
          ui=False,
          version='1.0',
          description="Retrieve IRIS from coordinate (latitude,longitude).")
apidoc.apidoc.static_url_path = service.url_prefix + apidoc.apidoc.static_url_path

coordinate_parser = api.parser()
coordinate_parser.add_argument("lon", required=True, dest='lon', location='args', help='Longitude')
coordinate_parser.add_argument("lat", required=True, dest='lat', location='args', help='Latitude')


iris_fields = api.model('Iris',
                        OrderedDict([('iris', fields.String),
                                     ('city', fields.String),
                                     ('citycode', fields.String),
                                     ('name', fields.String),
                                     ('complete_code', fields.String),
                                     ('type', fields.String)]))

@service.route('/doc/', endpoint='doc')
def swagger_ui():
    return apidoc.ui_for(api)


@api.route("/iris/<string:code>")
class IrisCode(Resource):
    @api.doc(description="get data for a specific IRIS")
    def get(self, code):
        # check the search_id
        iris = extract.get_iris_field(code)
        if not iris:
            api.abort(404, "IRIS code '{}' not found".format(code))
        return iris


@api.route("/search/")
class SearchIris(Resource):
    @api.doc(parser=coordinate_parser,
             description="Look for an IRIS for a coordinate (lon/lat).")
    @api.marshal_with(iris_fields, envelope='iris')
    def get(self):
        args = coordinate_parser.parse_args()
        lon, lat = args['lon'], args['lat']
        Logger.info("Look for IRIS from coordinate lon/lat ('%s', '%s')", lon, lat)
        return extract.iris_from_coordinate(lon, lat)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import io
import os
import logging
from datetime import datetime, date

from flask import Flask
from flask.json import JSONEncoder
from yaml import load as yload

from pyris.api.app import service

# app version
__version__ = '0.1.dev0'

ISO_FORMAT = "%Y-%m-%d"
Logger = logging.getLogger(__name__)

LOG_LEVELS = {
    'debug': logging.DEBUG,
    'info': logging.INFO,
    'warning': logging.WARNING,
    'error': logging.ERROR,
    'critical': logging.CRITICAL
}

console = logging.StreamHandler()
log_formatter = logging.Formatter("[%(asctime)s][%(levelname)s][%(module)s] %(message)s")
console.setFormatter(log_formatter)
Logger.addHandler(console)
# Log file
log_file = os.path.join(os.environ['HOME'], 'pyris-flaskapp.log')
fileHandler = logging.FileHandler(log_file)
fileHandler.setFormatter(log_formatter)
Logger.addHandler(fileHandler)


class CustomJSONEncoder(JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (date, datetime)):
            return obj.strftime(ISO_FORMAT)
        return JSONEncoder.default(self, obj)


def set_level(level='info'):
    """
    Set log level
    """
    Logger.setLevel(LOG_LEVELS.get(level))


def register_extensions(app):
    """
    Register extensions into flask application
    """
    app.register_blueprint(service)


def load_yaml_config(filename):
    """
    Open Yaml file, load content for flask config and returns it as a python dict
    """
    content = io.open(filename, 'r').read()
    return yload(content).get('flask', {})


def create_app(env='Defaults'):
    """
    Creates application.
    :returns: flask application instance
    """
    app = Flask(__name__)
    cfgfile = os.environ.get('PYRIS_APP_SETTINGS')
    print(cfgfile)
    Logger.info(cfgfile)
    if cfgfile:
        app.config.update(load_yaml_config(cfgfile))

    set_level(app.config['LOG_LEVEL'])
    Logger.info("Run the app")
    register_extensions(app)
    app.json_encoder = CustomJSONEncoder
    app.config.SWAGGER_UI_DOC_EXPANSION = 'list'

    return app



# coding: utf-8

"""Store some SQL queries
"""


query_iris = """SELECT
   iris
  ,nom_com
  ,depcom
  ,nom_iris
  ,dcomiris
  ,typ_iris
FROM geoiris
WHERE iris = %s
LIMIT 1;"""


# coordinate: (lon, lat)
query_coordinate = """SELECT
   iris
  ,nom_com
  ,depcom
  ,nom_iris
  ,dcomiris
  ,typ_iris
FROM geoiris
WHERE geom && st_makepoint(%s, %s)
LIMIT 1;"""



# coding: utf-8

"""Extract data from the database
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import logging

import psycopg2

from pyris.config import DATABASE
from pyris.api.sqlrequests import (query_iris,
                                   query_coordinate)


Logger = logging.getLogger(__name__)



def _query(q, params=None):
    """Carry out a SQL query

    Only fetch one result
    """
    with psycopg2.connect(database="pyris",
                          user=DATABASE['USER']) as cnx:
        with cnx.cursor() as cu:
            if params is not None:
                cu.execute(q, params)
            else:
                cu.execute(q)
            return cu.fetchone()


def _iris_fields(res):
    """Iris field from a SQL query result
    """
    return {"iris": res[0],
            'city': res[1],
            'citycode': res[2],
            'name': res[3],
            'complete_code': res[4],
            'type': res[5]}


def get_iris_field(code):
    """Get some date from the IRIS code
    """
    res = _query(query_iris, (code,))
    if res is not None:
        return _iris_fields(res)
    return res


def iris_from_coordinate(lon, lat):
    """Get the IRIS code from a coordinate.
    """
    res = _query(query_coordinate, (lon, lat))
    if res is not None:
        return _iris_fields(res)
    return res



# coding: utf-8

"""Pyris configuration

Retrieve user and password from the YAML configuration file for the database
access
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import os
import io

from yaml import load as yload


_cfgfile = os.environ.get('PYRIS_APP_SETTINGS')

with io.open(_cfgfile, 'r') as fobj:
    DATABASE = yload(fobj.read()).get('database', {})



# -*- coding: utf-8 -*-

import setuptools


with open("README.md") as fobj:
    LONG_DESCRIPTION = fobj.read()


setuptools.setup(
    name='pyris',
    version='0.1',
    license='BSD',
    url='https://github.com/garaud/pyris',
    packages=setuptools.find_packages(exclude=['data', 'app.yml']),
    include_package_data=True,
    install_requires=["flask", "flask-restplus", "pyaml"],

    author="Damien Garaud",
    author_email='damien.garaud@gmail.com',
    description="INSEE/IRIS geolocalization",
    long_description=LONG_DESCRIPTION,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
    ]
)
